// ---------------------------------------------------------------------------
// DLI Example Ethernet Power Controller switching program
// Requires: Borland RAD Studio 2007
// Indy v10.0 Indy free and is available free from http://www.indyproject.org/.
// I have not written the code to reset all settings on the EPCR
//
// 7/12/08 Added EPCR2 - V1.2.9 compatibailty
// 2/28/14 Renaming program (adding a number ex:epc_example1.exe) changes settings saving location
// 10/08/14 Added support for inverting On-Off on DIN relays
// Michael Gillen, Digital Loggers, Inc.
//
// ---------------------------------------------------------------------------
#include <vcl.h>
#include <Registry.hpp>
#pragma hdrstop
#include <IdMultipartFormData.hpp>
#include "Main.h"
#include "HourGlass.h"
#include "Defines.h"
// ---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TForm1 * Form1;

// ---------------------------------------------------------------------------
// Constructor
// ---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent * Owner) : TForm(Owner) {
	// If the filename has a number on the end, get the settings location
	Base_Key = BASE_KEY;
	String TheName = ChangeFileExt(ExtractFileName(Application->ExeName), L"");
	String Number;
	for (int i = 0; i < TheName.Length(); ++i) {
		if (isdigit(TheName.c_str()[i])) {
			Number = Number + String(TheName.c_str()[i]);
		}
	}
	Base_Key = Base_Key + Number;
	// Retreive settings from the registry
	EpcType = epcUNKNOWN;
//	LoadSettings();
//	DisableButtons();
	CbxInvertState->Visible = false;
	PostMessage(this->Handle, CM_APPSTART, 0, 0);
	if (FindCmdLineSwitch(L"DLI")) {
		BtnResetNames->Visible = true;
		BtnResetAll->Visible = true;
	}
	else {
		BtnResetNames->Visible = false;
		BtnResetAll->Visible = false;
	}

}

// ---------------------------------------------------------------------------
// The Application is constructed and is now processing messages
// Set up everything
// ---------------------------------------------------------------------------
void __fastcall TForm1::AppStart(TMessage & Message) {
	Application->ProcessMessages();
	LoadSettings();
	DisableButtons();
	if (EdtURL->Text.Length() > 10 && EdtPassword->Text.Length()) {
		GetStatus();
	}
}

// ---------------------------------------------------------------------------
// User presses the Get Status button
// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnGetStatusClick(TObject * Sender) {
	IdHTTP->Disconnect();
	// Get the status of the switches
	ClearStatus();
	GetStatus();
}

// ---------------------------------------------------------------------------
// ---------------------------------------------------------------------------
bool __fastcall TForm1::SetUpHTTP() {
	StatusBar->SimpleText = "Setting up HTTP communications to controller";
	Application->ProcessMessages();
	try {
		// This doesn't really matter
		IdHTTP->AllowCookies = true;
		// These do matter
		IdHTTP->Request->BasicAuthentication = true;
		IdHTTP->Request->Username = EdtUser->Text;
		IdHTTP->Request->Password = EdtPassword->Text;
		IdHTTP->ConnectTimeout = CONNECT_TIMEOUT;
		// This overrides the default Mozilla 3 signature which will cause an error
		IdHTTP->Request->UserAgent =
			"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; Q312461; .NET CLR 1.0.3705)";
		IdHTTP->HandleRedirects = true;
		// Prevents the 302 error because of redirection

	}
	catch (Exception & E) {
		String Msg = E.Message + String(" SetUpHTTP");
		MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
		StatusBar->SimpleText = "Cannot find power controller";
		return false;
	}
	return true;
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ClearAuthentication() {
	IdHTTP->Disconnect();
	// Delete the Authentication
	if (IdHTTP->Request->Authentication) {
		IdHTTP->Request->Authentication->Reset();
		delete IdHTTP->Request->Authentication;
		IdHTTP->Request->Authentication = NULL;
	}
}

// ---------------------------------------------------------------------------
// Gets the status and names of the switches
// ---------------------------------------------------------------------------
void __fastcall TForm1::GetStatus() {
	EpcType = epcUNKNOWN;
	StatusBar->SimpleText = "Getting Switch Status";
	Application->ProcessMessages();
	// Check for a password
	if (EdtPassword->Text.Length() < 1) {
		MessageDlg("Please enter a password.", mtError,
		TMsgDlgButtons() << mbOK, 0);
		EdtPassword->SetFocus();
		return;
	}
	TGeneral_RAII_HourGlass hourGlass;
	// Set up authentication
	if (!SetUpHTTP()) {
		return;
	}
	// Login and get the web page
	try {
		// Get the page
		AnsiString S = AnsiString("http://") + EdtURL->Text + ":" + EdtPort->Text +
			AnsiString("/index.htm");
		AnsiString Resp;
		try {
			Resp = IdHTTP->Get(S);
		}
		catch (Exception & E) {
			if (E.Message.Pos("Forbidden")) {
				ClearAuthentication();
				ShowMessage("Invalid username or password");
				return;
			}
			if (E.Message.Pos("Not Found")) {
				ClearAuthentication();
				ShowMessage("Page Not Found");
				return;
			}
			StatusBar->SimpleText = E.Message;
		}
		// We're authorized now. Changing the user name will not work.
		// EdtUser->Enabled = false;
		// Remove newline characters for compatability between EPCR and LPC
		while (Resp.Pos("\n")) {
			Resp.Delete(Resp.Pos("\n"), 1);
		}
		if (Resp.Length() < 30) {
			ClearAuthentication();
			return;
		}
		UpdateStatus(Resp);
		// Start with an unknown type
		EpcType = epcUNKNOWN;
		// if (Resp.Pos("href=outlet?1=OFF") || Resp.Pos("href=outlet?1=ON"))   *** MG
		// EpcType = epcLPC1; // This will catch old LPCs
		if (Resp.Pos("outletoff?1") || Resp.Pos("outleton?1")) {
			EpcType = epcPC8000;
		}
		// if (Resp.Pos("bgcolor=\"#ddccdd\">Bus A:"))
		// EpcType = epcEPCR2;
	}
	catch (Exception & E) {
		ClearAuthentication();
		String Msg = E.Message + String(" GetStatus 1");
		MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
		return;
	}
	try {
		if (EpcType != epcPC8000) {
			AnsiString S = AnsiString("http://") + EdtURL->Text + ":" +
				EdtPort->Text + AnsiString("/support.htm");
			AnsiString Resp = IdHTTP->Get(S);
			int maxJumperPos = Resp.Pos("Hardware");
			if (maxJumperPos) {
				S = Resp.SubString(maxJumperPos, 200);
				Resp = S;
				maxJumperPos = 200;
				if (Resp.Pos(JUMPER_48) > 10 && Resp.Pos(JUMPER_48) < maxJumperPos) {
					EpcType = epc48;
				}
				if (Resp.Pos(JUMPER_DIN) > 10 && Resp.Pos(JUMPER_DIN) < maxJumperPos) {
					// Jumpers are the same - memory is the difference
					S = AnsiString("http://") + EdtURL->Text + ":" + EdtPort->Text +
						AnsiString("/support.htm");
					Resp = IdHTTP->Get(S);
					maxJumperPos = Resp.Pos("memory");
					S = Resp;
					if (maxJumperPos) {
						int i = S.Pos("KiB");
						if (i) {
							AnsiString AS = S;
							AnsiString SS = AS.SubString(i - 4, 3);
							if (!isdigit(SS.c_str()[0])) {
								SS = S.SubString(i - 3, 3);
							}
							SS = Trim(SS);
							int mem = StrToIntDef(SS, 0);
							if (mem) {
								if (mem >= 32) {
									EpcType = epcDIN2;
								}
								else {
									if (mem >= 16) {
										EpcType = epcDIN;
									}
								}
							}
						}
					}
				}

			}
			if (Resp.Pos(JUMPER_EPCR3) > 10 && Resp.Pos(JUMPER_EPCR3) < maxJumperPos)
			{
				EpcType = epcEPCR3;
			}
			if (Resp.Pos(JUMPER_VPDU) > 10 && Resp.Pos(JUMPER_VPDU) < maxJumperPos) {
				EpcType = epcVPDU;
			}
			if (Resp.Pos(JUMPER_EPCR2) > 10 && Resp.Pos(JUMPER_EPCR2) < maxJumperPos)
			{
				EpcType = epcEPCR2;
			}
			// VPDU and LPC3 look almost the same
			if (EpcType != epcVPDU && Resp.Pos(JUMPER_LPC3) >
				10 && Resp.Pos(JUMPER_LPC3) < maxJumperPos) {
				// Jumpers are the same - memory is the difference
				S = AnsiString("http://") + EdtURL->Text + ":" + EdtPort->Text +
					AnsiString("/support.htm");
				Resp = IdHTTP->Get(S);
				maxJumperPos = Resp.Pos("memory");
				S = Resp;
				if (maxJumperPos) {
					int i = S.Pos("KiB");
					if (i) {
						AnsiString AS = S;
						AnsiString SS = AS.SubString(i - 4, 3);
						if (!isdigit(SS.c_str()[0])) {
							SS = S.SubString(i - 3, 3);
						}
						SS = Trim(SS);
						int mem = StrToIntDef(SS, 0);
						if (mem) {
							if (mem >= 32) {
								EpcType = epcLPC3;
							}
							else {
								if (mem >= 16) {
									EpcType = epcLPC2;
								}
							}
						}
					}
				}
			}
			if (Resp.Pos(JUMPER_DIN3) > 10 && Resp.Pos(JUMPER_DIN3) < maxJumperPos) {
				EpcType = epcDIN3;
			}
			if (Resp.Pos(JUMPER_LPC4) > 10 && Resp.Pos(JUMPER_LPC4) < maxJumperPos) {
				EpcType = epcLPC4;
			}
			if (Resp.Pos(JUMPER_LPC5) > 10 && Resp.Pos(JUMPER_LPC5) < maxJumperPos) {
				EpcType = epcLPC5;
			}
			if (Resp.Pos(JUMPER_LPC6) > 10 && Resp.Pos(JUMPER_LPC6) < maxJumperPos) {
				EpcType = epcLPC6;
			}
			if (Resp.Pos(JUMPER_LPC7) > 10 && Resp.Pos(JUMPER_LPC7) < maxJumperPos) {
				EpcType = epcLPC7;
			}
			if (Resp.Pos(JUMPER_POE15) > 10 && Resp.Pos(JUMPER_POE15) < maxJumperPos)
			{
				EpcType = epcPOE15;
			}
			if (Resp.Pos(JUMPER_POE24) > 10 && Resp.Pos(JUMPER_POE24) < maxJumperPos)
			{
				EpcType = epcPOE24;
			}
			if (Resp.Pos(JUMPER_POER24) > 10 && Resp.Pos(JUMPER_POER24) <
				maxJumperPos) {
				EpcType = epcPOER24;
			}
			if (Resp.Pos(JUMPER_POE48) > 10 && Resp.Pos(JUMPER_POE48) < maxJumperPos)
			{
				EpcType = epcPOE48;
			}
			if (Resp.Pos(JUMPER_220) > 10 && Resp.Pos(JUMPER_220) < maxJumperPos) {
				EpcType = epc220;
			}
		}
		if (EpcType == epcUNKNOWN) {
			// Older power controller firmware
			String S = AnsiString("http://") + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/register.htm");
			String Resp = IdHTTP->Get(S);
			int maxJumperPos = Resp.Pos("Version</td>"); // version string
			if (maxJumperPos) {
				S = Resp.SubString(maxJumperPos, 400);
				Resp = S;
				maxJumperPos = 400;
				if (Resp.Pos(JUMPER_LPC3) > 10 && Resp.Pos(JUMPER_LPC3) < maxJumperPos)
				{
					EpcType = epcLPC3;
				}
				if (Resp.Pos(JUMPER_LPC4) > 10 && Resp.Pos(JUMPER_LPC4) < maxJumperPos)
				{
					EpcType = epcLPC4;
				}
				if (Resp.Pos(JUMPER_EPCR2) > 10 && Resp.Pos(JUMPER_EPCR2) <
					maxJumperPos) {
					EpcType = epcEPCR2;
				}

			}
			if (EpcType == epcUNKNOWN) {
				// Very old power controller
				S = AnsiString("http://") + EdtURL->Text + ":" + EdtPort->Text +
					AnsiString("/register.htm");
				Resp = IdHTTP->Get(S);
				maxJumperPos = Resp.Pos("Installed");
				// Installed Memory string
				if (maxJumperPos < 1) {
					maxJumperPos = 3000;
				}
				if (Resp.Pos(JUMPER_NONE) > 500 && Resp.Pos(JUMPER_NONE) < maxJumperPos)
				{
					String S = Resp.SubString(maxJumperPos, 1000);
					int i = S.Pos("KiB");
					if (i) {
						AnsiString AS = S;
						AnsiString SS = AS.SubString(i - 4, 3);
						if (!isdigit(SS.c_str()[0])) {
							SS = S.SubString(i - 3, 3);
						}
						SS = Trim(SS);
						int mem = StrToIntDef(SS, 0);
						if (mem) {
							if (mem >= 32) {
								EpcType = epcLPC3;
							}
							else {
								if (mem >= 16) {
									EpcType = epcLPC2;
								}
							}
						}
					}
				}
			}
		}

	}
	catch (Exception & E) {
		ClearAuthentication();
		String Msg = E.Message + String(" Type Detect");
		MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
		return;
	}
	IdHTTP->Disconnect();

	if (EpcType != epcUNKNOWN) {
		BtnResetNames->Enabled = true;
	}
	else {
		BtnResetNames->Enabled = false;
	}
	BtnAllOn->Enabled = BtnResetNames->Enabled;
	BtnAllOff->Enabled = BtnResetNames->Enabled;

	if (EpcType != epcUNKNOWN && EpcType != epcPC8000) {
		BtnResetAll->Enabled = BtnResetNames->Enabled;
	}
	SetCaption(EpcType);
	if(CanInvertOnOff)
	{
		CbxInvertState->Visible = true;
	}
	else
	{
		CbxInvertState->Visible = false;
		CbxInvertState->Checked = false;
	}
	StatusBar->SimpleText = "Ready";
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
// Captions for the controller models
// ---------------------------------------------------------------------------
void __fastcall TForm1::SetCaption(EpcTypes EpcType) {
	StatusBar->SimpleText = "Setting Captions";
	Application->ProcessMessages();
	LblEpcType->Caption = CAPTION_TYPE_UNKNOWN;
	CanInvertOnOff = false;
	switch (EpcType) {
	case epcPC8000:
		LblEpcType->Caption = CAPTION_TYPE_PC8000;
		break;

	case epcLPC1:
		LblEpcType->Caption = CAPTION_TYPE_LPC1;
		break;
	case epcLPC2:
		LblEpcType->Caption = CAPTION_TYPE_LPC2;
		break;
	case epcLPC3:
		LblEpcType->Caption = CAPTION_TYPE_LPC3;
		break;
	case epcLPC4:
		LblEpcType->Caption = CAPTION_TYPE_LPC4;
		break;
	case epcLPC5:
		LblEpcType->Caption = CAPTION_TYPE_LPC5;
		break;
	case epcLPC6:
		LblEpcType->Caption = CAPTION_TYPE_LPC6;
		break;
	case epcLPC7:
		LblEpcType->Caption = CAPTION_TYPE_LPC7;
		break;

	case epcEPCR2:
		LblEpcType->Caption = CAPTION_TYPE_EPCR2;
		break;
	case epcEPCR3:
		LblEpcType->Caption = CAPTION_TYPE_EPCR3;
		break;

	case epcVPDU:
		LblEpcType->Caption = CAPTION_TYPE_VPDU;
		break;

	case epcDIN:
		LblEpcType->Caption = CAPTION_TYPE_DIN;
		CanInvertOnOff = true;
		break;
	case epcDIN2:
		LblEpcType->Caption = CAPTION_TYPE_DIN2;
		CanInvertOnOff = true;
		break;
	case epcDIN3:
		LblEpcType->Caption = CAPTION_TYPE_DIN3;
		CanInvertOnOff = true;
		break;

	case epc48:
		LblEpcType->Caption = CAPTION_TYPE_48;
		break;

	case epcPOE15:
		LblEpcType->Caption = CAPTION_TYPE_POE15;
		break;
	case epcPOE24:
		LblEpcType->Caption = CAPTION_TYPE_POE24;
		break;
	case epcPOER24:
		LblEpcType->Caption = CAPTION_TYPE_POER24;
		break;
	case epcPOE48:
		LblEpcType->Caption = CAPTION_TYPE_POE48;
		break;

	case epc220:
		LblEpcType->Caption = CAPTION_TYPE_220;
		break;

	}

}

// ---------------------------------------------------------------------------
// Disable buttons
// ---------------------------------------------------------------------------
void __fastcall TForm1::DisableButtons() {
	BtnResetAll->Enabled = false;
	BtnResetNames->Enabled = false;
	BtnAllOn->Enabled = false;
	BtnAllOff->Enabled = false;
}

// ---------------------------------------------------------------------------
// Clear the status if we don't know anything
// ---------------------------------------------------------------------------
void __fastcall TForm1::ClearStatus() {
	EpcType = epcUNKNOWN;
	DisableButtons();
	TSpeedButton * Bp;
	TLabel * Lb;
	// Look through all of the components to find what we need
	for (int i = 0; i < Form1->ComponentCount; ++i) {
		Bp = dynamic_cast<TSpeedButton*>(Form1->Components[i]);
		// For the buttons
		Lb = dynamic_cast<TLabel*>(Form1->Components[i]);
		// For the switch names
		// We have a label
		if (Lb) {
			if (Lb->Name == "LabelControllerName") {
				Lb->Caption = CAPTION_TYPE_UNKNOWN;
			}
			for (int i = 1; i < 9; ++i) // Set the names of the switches
			{
				AnsiString S = "Label" + IntToStr(i);
				if (Lb->Name == S) { // Assign default names
					Lb->Caption = "Switch " + IntToStr(i);
				}
			}
		}
		// We have a switch button
		if (Bp) {
			for (int i = 1; i < 9; ++i) // Reset the status of the switches
			{
				// Generate the switch button name
				AnsiString S = "SwitchButton" + IntToStr(i);
				if (Bp->Name == S)
					// If it's the correct button, display the switch status
				{
					Bp->Font->Style = Bp->Font->Style << fsBold;
					Bp->Font->Color = clBlack;
					Bp->Caption = "?";
				}
			}
		}
	}
	UpdateCycleStatus();
	StatusBar->SimpleText = "Ready";
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
// Make the cycle buttons visible or not
// ---------------------------------------------------------------------------
void __fastcall TForm1::UpdateCycleStatus() {
	TSpeedButton * Sb, *Cb;
	for (int i = 0; i < Form1->ComponentCount; ++i) {
		Sb = dynamic_cast<TSpeedButton*>(Form1->Components[i]);
		// For the buttons
		// We have a button
		if (Sb) {
			for (int i = 1; i < 9; ++i) // Display the status of the switches
			{
				// Generate the switch button name
				AnsiString SbName = "SwitchButton" + IntToStr(i);
				if (Sb->Name == SbName) // If it's the correct button
				{
					for (int j = 0; j < Form1->ComponentCount; ++j) {
						Cb = dynamic_cast<TSpeedButton*>(Form1->Components[j]);
						if (Cb) {
							AnsiString CbName = "CycleButton" + IntToStr(i);
							if (Cb->Name == CbName)
								// If it's the correct button
							{
								Cb->Width = 50;
								Cb->Caption = "Cycle";
								Cb->Enabled = true;
								if (CbxInvertState->Checked) {
									Cb->Visible = Sb->Caption == "off" || !Sb->Enabled;
									if (Cb->Visible && !Sb->Enabled) {
										Cb->Caption = "Locked";
										Cb->Enabled = false;
									}

								}
								else {
									Cb->Visible = Sb->Caption == "on" || !Sb->Enabled;
									if (Cb->Visible && !Sb->Enabled) {
										Cb->Caption = "Locked";
										Cb->Enabled = false;
									}
								}
							}
						}
					}
				}
			}
		} // Sb
	}
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
// Update the status of the controller
// ---------------------------------------------------------------------------
void __fastcall TForm1::UpdateStatus(const AnsiString & Resp) {
	ClearStatus();
	if (Resp.Length() < 30) {
		return;
	}
	int pos = Resp.Pos("<!-- state");
	String Temp = Resp.SubString(pos, 30);
	String TheState;
	String TheLocks;
	pos = Temp.Pos("state=");
	if (pos) {
		TheState = Temp.SubString(pos + 6, 2);
		// StrToHex
	}
	pos = Temp.Pos("lock=");
	if (pos) {
		TheLocks = Temp.SubString(pos + 5, 2);
	}
	TSpeedButton * Bp;

	TLabel * Lb;
	// Look through all of the components to find what we need
	for (int i = 0; i < Form1->ComponentCount; ++i) {
		Bp = dynamic_cast<TSpeedButton*>(Form1->Components[i]);
		// For the buttons
		Lb = dynamic_cast<TLabel*>(Form1->Components[i]);
		// For the switch names
		// We have a label
		if (Lb) {
			if (Lb->Name == "LabelControllerName") {
				AnsiString Flag = AnsiString("Controller:");
				AnsiString Name;
				if (Resp.Pos(Flag)) // We found it
				{
					Name = Resp.SubString(Resp.Pos(Flag) + 11, 50);
					Name = Name.SubString(1, Name.Pos("</th>") - 1);
					// Find the end
				}
				Lb->Caption = Trim(Name);
				continue;
			}
			// Get the names of the switches
			for (int i = 1; i < 9; ++i) {
				// This text is before the switch name on the web page
				AnsiString Flag = AnsiString("ter>") + IntToStr(i) + "</td><td>";
				AnsiString Name;
				if (Resp.Pos(Flag)) // We found it
				{
					Name = Resp.SubString(Resp.Pos(Flag) + 14, 50);
					Name = Name.SubString(1, Name.Pos("</t") - 1);
					// Find the end
					if (Name.Length() > 27) { // Trim it if it's too long
						Name.c_str()[27] = 0;
					}
				}
				// Generate the name of the label
				AnsiString S = "Label" + IntToStr(i);
				if (Lb->Name == S) {
					// Assign the switch name to the appropriate label caption
					Lb->Caption = Name;
				}
			}
		}

		// We have a switch button
		if (Bp) {
			// See if the switch is on or off
			for (int i = 1; i < 9; ++i) {
				// Generate the switch button name
				AnsiString S = "SwitchButton" + IntToStr(i);
				if (Bp->Name == S)
					// If it's the correct button, display the switch status
				{
					Bp->Font->Style = Bp->Font->Style << fsBold;
					Bp->Font->Color = clBlack;
					Bp->Enabled = true;
					// We'll parse the state
					int part = 0;
					// If the switch is 5-8 we use left number (part 0), 1-4 are the right number (part 1)
					if (i < 5) {
						part = 1;
					}
					// This item is in hex, so I need to handle A-F too.
					int iTheState =
						StrToInt(String("0x") + String(TheState.c_str()[part]));
					int position = 1 << i - 1;
					if (i > 4) {
						position = 1 << (i - 5);
					}
					if (position & iTheState) {
						if (CbxInvertState->Checked) {
							Bp->Font->Color = clRed;
							Bp->Caption = "off";
						}
						else {
							Bp->Font->Color = clGreen;
							Bp->Caption = "on";
						}
					}
					else {
						if (CbxInvertState->Checked) {
							Bp->Font->Color = clGreen;
							Bp->Caption = "on";
						}
						else {
							Bp->Font->Color = clRed;
							Bp->Caption = "off";
						}
					}

					// We'll parse the lock state on EPCR controllers
					part = 0;
					// If the switch is 5-8 we use left number (part 0), 1-4 are the right number (part 1)
					if (i < 5) {
						part = 1;
					}
					// This item is in hex, so I need to handle A-F too.
					int iTheLocks =
						StrToInt(String("0x") + String(TheLocks.c_str()[part]));
					position = 1 << i - 1;
					if (i > 4) {
						position = 1 << (i - 5);
					}
					if (position & iTheLocks) {
						Bp->Enabled = false;
					}
				}
			}
		} // Bp
	}
	UpdateCycleStatus();
}

// ---------------------------------------------------------------------------
// Bye bye...
// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnExitClick(TObject * Sender) {
	Close();
}

// ---------------------------------------------------------------------------
// Save the settings to the registry
// ---------------------------------------------------------------------------
void __fastcall TForm1::SaveSettings() {
	StatusBar->SimpleText = "Saving Settings";
	Application->ProcessMessages();
	TRegistry * Reg = new TRegistry();
	Reg->RootKey = HKEY_CURRENT_USER;
	try {
		if (Reg->OpenKey(Base_Key, true)) {
			Reg->WriteString("User", EdtUser->Text);
			Reg->WriteString("URL", EdtURL->Text);
			Reg->WriteString("Port", EdtPort->Text);
			Reg->WriteBool("InvertState", CbxInvertState->Checked);
#ifdef SAVE_PASSWORD
			// WARNING! Plain Text
			Reg->WriteString("Pass", EdtPassword->Text);
#endif
			Reg->WriteString("Script 1 Description", EdtScriptDesc1->Text);
			Reg->WriteString("Script 2 Description", EdtScriptDesc2->Text);
			Reg->WriteString("Script 3 Description", EdtScriptDesc3->Text);
			Reg->WriteString("Script 1 Line", EdtScriptLine1->Text);
			Reg->WriteString("Script 2 Line", EdtScriptLine2->Text);
			Reg->WriteString("Script 3 Line", EdtScriptLine3->Text);
		}
	}
	catch (Exception & E) {
		String Msg = E.Message + String(" - Save Settings");
		MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
	}
	delete Reg;
}

// ---------------------------------------------------------------------------
// Load the settings from the registry
// ---------------------------------------------------------------------------
void __fastcall TForm1::LoadSettings() {
	TRegistry * Reg = new TRegistry();
	Reg->RootKey = HKEY_CURRENT_USER;
	try {
		if (Reg->OpenKey(Base_Key, true)) {
			EdtUser->Text = Reg->ReadString("User");
			if (!EdtUser->Text.Length()) {
				EdtUser->Text = "admin";
			}
			EdtURL->Text = Reg->ReadString("URL");
			EdtPort->Text = Reg->ReadString("Port");
			try
			{
				CbxInvertState->Checked = Reg->ReadBool("InvertState");
			}
			catch (Exception &E)
			{
				CbxInvertState->Checked = false;
			}
#ifdef SAVE_PASSWORD
			EdtPassword->Text = Reg->ReadString("Pass");
			IdHTTP->Request->Password = EdtPassword->Text;
#endif
			EdtScriptDesc1->Text = Reg->ReadString("Script 1 Description");
			EdtScriptDesc2->Text = Reg->ReadString("Script 2 Description");
			EdtScriptDesc3->Text = Reg->ReadString("Script 3 Description");
			EdtScriptLine1->Text = Reg->ReadString("Script 1 Line");
			EdtScriptLine2->Text = Reg->ReadString("Script 2 Line");
			EdtScriptLine3->Text = Reg->ReadString("Script 3 Line");

			if (!EdtScriptDesc1->Text.Length()) {
				EdtScriptDesc1->Text = "Script 1";
			}
			if (!EdtScriptDesc2->Text.Length()) {
				EdtScriptDesc2->Text = "Script 2";
			}
			if (!EdtScriptDesc3->Text.Length()) {
				EdtScriptDesc3->Text = "Script 3";
			}
			if (EdtScriptLine1->Text.Length() < 3) {
				EdtScriptLine1->Text = "002";
			}
			if (EdtScriptLine2->Text.Length() < 3) {
				EdtScriptLine2->Text = "010";
			}
			if (EdtScriptLine3->Text.Length() < 3) {
				EdtScriptLine3->Text = "015";
			}

		}
	}
	catch (Exception & E) {
		String Msg = E.Message + String(" - Load Settings");
		MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
	}
	delete Reg;
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetPowerRecovery() {
	TGeneral_RAII_HourGlass hourGlass;
	TStringList * PostData = new TStringList;
	try {
		PostData->Add("recmode=0");
		try {
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/powloss.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST: " + e.Message);
		}
	}
	__finally {
		delete PostData;
	}
	::Sleep(500);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetLinks() {
	TGeneral_RAII_HourGlass hourGlass;
	TStringList * PostData = new TStringList;
	try {

		if (EpcType != epcPC8000 && EpcType != epcUNKNOWN) {
			PostData->Add("lnkdesc1=Manual");
			PostData->Add("lnkdesc2=FAQ");
			PostData->Add("lnkdesc3=Product Information");
			PostData->Add("lnkdesc4=Digital Loggers, Inc.");
		}
		switch (EpcType) {
		case epcPC8000:
			PostData->Add("lnkhref1=http://www.digital-loggers.com");
			PostData->Add("lnkdesc1=Link 1");
			PostData->Add("lnkhref2=http://www.digital-loggers.com");
			PostData->Add("lnkdesc2=Link 2");
			PostData->Add("lnkhref3=http://www.digital-loggers.com");
			PostData->Add("lnkdesc3=Link 3");
			PostData->Add("lnkhref4=http://www.digital-loggers.com");
			PostData->Add("lnkdesc4=Link 4");
			break;

		case epcEPCR2:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/1.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/2.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/3.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/4.html");
			break;
		case epcLPC1:
		case epcLPC2:
		case epcLPC3:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/5.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/6.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/7.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/8.html");
			break;
		case epcLPC4:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/P.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/Q.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/R.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/S.html");
			break;
		case epcLPC5:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/12.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/13.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/14.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/15.html");
			break;
		case epcLPC6:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/1E.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/1F.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/1G.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/1H.html");
			break;
		case epcLPC7:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/1Q.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/1R.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/1S.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/1T.html");
			break;
		case epcDIN:
		case epcDIN2:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/9.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/A.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/B.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/C.html");
			break;
		case epcDIN3:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/1M.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/1N.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/1O.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/1P.html");
			break;
		case epc48:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/D.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/E.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/F.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/G.html");
			break;
		case epcVPDU:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/H.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/I.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/J.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/K.html");
			break;
		case epcEPCR3:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/L.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/M.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/N.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/O.html");
			break;

		case epcPOE15:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/X.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/Y.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/Z.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/0.html");
			break;

		case epcPOE24:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/16.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/17.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/18.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/19.html");
			break;

		case epcPOER24:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/1I.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/1J.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/1K.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/1L.html");
			break;

		case epcPOE48:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/T.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/U.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/V.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/W.html");
			break;

		case epc220:
			PostData->Add("lnkhref1=http://www.digital-loggers.com/T.html");
			PostData->Add("lnkhref2=http://www.digital-loggers.com/U.html");
			PostData->Add("lnkhref3=http://www.digital-loggers.com/V.html");
			PostData->Add("lnkhref4=http://www.digital-loggers.com/W.html");
			break;

		}

		try {
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/userlinks.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST Links: " + e.Message);
		}
	}
	__finally {
		delete PostData;
	}
	::Sleep(500);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetDelay() {
	TGeneral_RAII_HourGlass hourGlass;
	TStringList * PostData = new TStringList;
	try {
		PostData->Add("lockdel=3");
		PostData->Add("seqdelay=10");
		PostData->Add("refdelay=5");
		PostData->Add("refresh=checked");
		try {
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/delay.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST ResetDelay:" + e.Message);
		}
	}
	__finally {
		delete PostData;
	}

	::Sleep(500);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetNetwork() {
	TGeneral_RAII_HourGlass hourGlass;
	TStringList * PostData = new TStringList;
	try {
		PostData->Add("httport=80");
		PostData->Add("ipaddr=192.168.0.100");
		PostData->Add("netmask=255.255.255.0");
		PostData->Add("gateway=192.168.0.1");
		PostData->Add("samenet=checked");
		// comment out this line for unhecked
		PostData->Add("syslog=0.0.0.0");
		try {
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/network.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST Reset Network: " + e.Message);
		}
	}
	__finally {
		delete PostData;
	}
	StatusBar->SimpleText = "Waiting for network...";
	Application->ProcessMessages();
	for (int i = 0; i < 40; ++i) {
		::Sleep(200);
		Application->ProcessMessages();
	}
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetOutletNames() {
	TGeneral_RAII_HourGlass hourGlass;
	TStringList * PostData = new TStringList;
	// LPC version
	try {
		switch (EpcType) {
		case epcPC8000:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_PC8000);
			break;
		case epcEPCR2:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_EPCR2);
			break;
		case epcLPC1:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC1);
			break;
		case epcLPC2:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC2);
			break;
		case epcLPC3:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC3);
			break;
		case epcLPC4:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC4);
			break;
		case epcLPC5:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC5);
			break;
		case epcLPC6:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC6);
		case epcLPC7:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_LPC7);
			break;
		case epcDIN:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_DIN);
			break;
		case epcDIN2:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_DIN2);
			break;
		case epcDIN3:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_DIN3);
			break;
		case epc48:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_POE48);
			break;
		case epcPOE15:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_POE15);
			break;
		case epcPOE24:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_POE24);
			break;
		case epcPOER24:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_POER24);
			break;
		case epcVPDU:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_VPDU);
			break;
		case epcEPCR3:
			PostData->Add(String("ctrlname=") + CAPTION_TYPE_EPCR3);
			break;
		}

		if (EpcType == epcPC8000 || EpcType == epcEPCR2 || EpcType == epcEPCR3) {
			PostData->Add("outname1=Outlet 1 - Bus A");
			PostData->Add("outname2=Outlet 2 - Bus A");
			PostData->Add("outname3=Outlet 3 - Bus A");
			PostData->Add("outname4=Outlet 4 - Bus A");
			PostData->Add("outname5=Outlet 5 - Bus B");
			PostData->Add("outname6=Outlet 6 - Bus B");
			PostData->Add("outname7=Outlet 7 - Bus B");
			PostData->Add("outname8=Outlet 8 - Bus B");
		}
		else if (EpcType >= epcPOE15)
			// Highest number (ID) first  - above DINs
		{
			PostData->Add("outname1=Port 1");
			PostData->Add("outname2=Port 2");
			PostData->Add("outname3=Port 3");
			PostData->Add("outname4=Port 4");
			PostData->Add("outname5=Port 5");
			PostData->Add("outname6=Port 6");
			PostData->Add("outname7=Port 7");
			PostData->Add("outname8=Port 8");

		}

		else if (EpcType > epcDIN) {
			PostData->Add("outname1=Relay 1");
			PostData->Add("outname2=Relay 2");
			PostData->Add("outname3=Relay 3");
			PostData->Add("outname4=Relay 4");
			PostData->Add("outname5=Relay 5");
			PostData->Add("outname6=Relay 6");
			PostData->Add("outname7=Relay 7");
			PostData->Add("outname8=Relay 8");

		}
		else {
			PostData->Add("outname1=Outlet 1");
			PostData->Add("outname2=Outlet 2");
			PostData->Add("outname3=Outlet 3");
			PostData->Add("outname4=Outlet 4");
			PostData->Add("outname5=Outlet 5");
			PostData->Add("outname6=Outlet 6");
			PostData->Add("outname7=Outlet 7");
			PostData->Add("outname8=Outlet 8");
		}

		try {
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/unitnames.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST Reset Outlets: " + e.Message);
		}
	}
	__finally {
		delete PostData;
	}
	::Sleep(500);
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnResetNamesClick(TObject * Sender) {
	if (MessageDlg("Reset all outlet names and links to default?", mtWarning,
		TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes) {
		StatusBar->SimpleText = "Resetting Outlet Names";
		Application->ProcessMessages();
		ResetOutletNames();
		StatusBar->SimpleText = "Resetting Links";
		Application->ProcessMessages();
		ResetLinks();
		GetStatus();
	}
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::AllOn() {
	TGeneral_RAII_HourGlass hourGlass;
	if (EpcType == epcPC8000) {
		IdHTTP->Get("http://" + EdtURL->Text + ":" + EdtPort->Text +
			AnsiString("/outleton?a="));
	}
	else {
		IdHTTP->Get("http://" + EdtURL->Text + ":" + EdtPort->Text +
			AnsiString("/outlet?a=ON"));
	}
	::Sleep(1000);
	GetStatus();
	::Sleep(2000);
	GetStatus();
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::AllOff() {
	TGeneral_RAII_HourGlass hourGlass;
	if (EpcType == epcPC8000) {
		IdHTTP->Get("http://" + EdtURL->Text + ":" + EdtPort->Text +
			AnsiString("/outletoff?a="));
	}
	else {
		IdHTTP->Get("http://" + EdtURL->Text + ":" + EdtPort->Text +
			AnsiString("/outlet?a=OFF"));
	}
	::Sleep(250);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnAllOnClick(TObject * Sender) {
	if (CbxInvertState->Checked)
		AllOff();
	else
		AllOn();
	GetStatus();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnAllOffClick(TObject * Sender) {
	if (CbxInvertState->Checked)
		AllOn();
	else
		AllOff();
	GetStatus();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::SwitchButtonClick(TObject * Sender) {
	// Don't let them click-click-click too fast
	if (Screen->Cursor == crHourGlass) {
		return;
	}
	TGeneral_RAII_HourGlass hourGlass;
	// When the switch button
	TSpeedButton * Bp;
	Bp = dynamic_cast<TSpeedButton*>(Sender);
	if (Bp) {
		if (Bp->Caption.Pos("unknown")) {
			MessageDlg("You must successfully obtain the status first.", mtError,
				TMsgDlgButtons() << mbOK, 0);
			return;
		}
		try {
			AnsiString S = "http://" + EdtURL->Text + ":" + EdtPort->Text + "/outlet";
			if (EpcType == epcPC8000) {
				if (Bp->Caption.Pos("on")) {
					S = S + "off?" + IntToStr(Bp->Tag);
				}
				else {
					S = S + "on?" + IntToStr(Bp->Tag);
				}
			}
			else {
				S = "http://" + EdtURL->Text + ":" + EdtPort->Text + "/outlet?";
				if (CbxInvertState->Checked) {
					if (Bp->Caption.Pos("on"))
						S = S + IntToStr(Bp->Tag) + AnsiString("=ON");
					else
						S = S + IntToStr(Bp->Tag) + AnsiString("=OFF");
				}
				else {
					if (Bp->Caption.Pos("on"))
						S = S + IntToStr(Bp->Tag) + AnsiString("=OFF");
					else
						S = S + IntToStr(Bp->Tag) + AnsiString("=ON");

				}
			}
			IdHTTP->Get(S);
			::Sleep(500);
			GetStatus();
		}
		catch (Exception & E) {
			ClearAuthentication();
			String Msg = E.Message + String(" - Switch Outlet");
			MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
		}
		IdHTTP->Disconnect();
	}
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnResetAllClick(TObject * Sender) {
	if (MessageDlg("Reset all settings to default?", mtWarning,
		TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes) {
		TGeneral_RAII_HourGlass hourGlass;
		DisableButtons();
		AllOff();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Outlet Names";
		Application->ProcessMessages();
		ResetOutletNames();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Power Recovery Options";
		Application->ProcessMessages();
		ResetPowerRecovery();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Links";
		Application->ProcessMessages();
		ResetLinks();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Delay and Refresh";
		Application->ProcessMessages();
		ResetDelay();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting AutoPing Targets";
		Application->ProcessMessages();
		ResetAutoPing();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Users";
		Application->ProcessMessages();
		ResetUsers();
		::Sleep(500);
		StatusBar->SimpleText = "Resetting Network";
		Application->ProcessMessages();
		ResetNetwork();
		StatusBar->SimpleText = "Done - Disconnecting";
		Application->ProcessMessages();
		ClearAuthentication();
		::Sleep(2000);
		ClearStatus();
	}
	::Sleep(500);
	GetStatus();
	Application->ProcessMessages();
}
// ---------------------------------------------------------------------------

void __fastcall TForm1::CycleButtonClick(TObject * Sender) {
	// Don't let them click-click-click too fast
	if (Screen->Cursor == crHourGlass) {
		return;
	}
	TGeneral_RAII_HourGlass hourGlass;
	// When the switch button
	TSpeedButton * CycleButton;
	CycleButton = dynamic_cast<TSpeedButton*>(Sender);
	if (CycleButton) {
		try {
			AnsiString S = "http://" + EdtURL->Text + ":" + EdtPort->Text + "/outlet";
			if (EpcType == epcPC8000) {
				S = "http://" + EdtURL->Text + ":" + EdtPort->Text + "/outletgl?" +
					IntToStr(CycleButton->Tag);
			}
			else {
				S = "http://" + EdtURL->Text + ":" + EdtPort->Text + "/outlet?" +
					IntToStr(CycleButton->Tag) + AnsiString("=CCL");
			}

			IdHTTP->Get(S);
			::Sleep(1000);
			GetStatus();
			::Sleep(2000);
			GetStatus();
		}
		catch (Exception & E) {
			String Msg = E.Message + String(" - Cycle Outlet");
			MessageDlg(Msg, mtError, TMsgDlgButtons() << mbOK, 0);
		}
		IdHTTP->Disconnect();
	}
}
// ---------------------------------------------------------------------------

void __fastcall TForm1::FormClose(TObject * Sender, TCloseAction & Action) {
	IdHTTP->Disconnect();
	SaveSettings();
}

// ---------------------------------------------------------------------------
// Clear the additional users
// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetUsers() {
	TGeneral_RAII_HourGlass hourGlass;

	TStringList * PostData = new TStringList;
	try {
		for (int i = 0; i < 8; ++i) {
			try {
				String S = IntToStr(i);
				PostData->Clear();
				PostData->Add("login=admin");
				PostData->Add("password=");
				PostData->Add(String("uid=") + S);
				PostData->Add("submit=Change");
				IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
					AnsiString("/users.cgi"), PostData);
			}
			catch (const Exception & e) {
				ShowMessage("Error encountered during POST Reset Users: " + e.Message);
			}
			::Sleep(500);
			Application->ProcessMessages();
		}
		try {
			PostData->Clear();
			PostData->Add("submit=Submit");
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/usersets.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST: " + e.Message);
		}

	}
	__finally {
		delete PostData;
	}
	::Sleep(500);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
// Clear the auto-pings and reset defaults
// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetAutoPing() {
	TGeneral_RAII_HourGlass hourGlass;
	if (!IdHTTP->Connected()) {
		if (!SetUpHTTP()) {
			return;
		}
	}
	TStringList * PostData = new TStringList;
	try {
		// Delete the auto-ping IP addresses
		for (int i = 0; i < 8; ++i) {
			try {
				String S = IntToStr(i);
				PostData->Clear();
				PostData->Add("pid=0");
				// Each delete will make the first pid zero
				PostData->Add("del=Delete");
				IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
					AnsiString("/autoping.cgi"), PostData);
			}
			catch (const Exception & e) {
				ShowMessage("Error encountered during POST Reset AutoPing: " +
					e.Message);
			}
			::Sleep(500);
			Application->ProcessMessages();
		}
		// Reset other settings to default
		try {
			PostData->Clear();
			PostData->Add("pint=120");
			PostData->Add("pthr=5");
			PostData->Add("ptrl=10");
			PostData->Add("prtr=1");
			PostData->Add("pdel=300");
			PostData->Add("submit=Apply");
			IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/autoping.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST Reset Autoping Settings: " +
				e.Message);
		}
	}

	__finally {
		delete PostData;
	}
	::Sleep(500);
	Application->ProcessMessages();
}

// ---------------------------------------------------------------------------
// Clear scripts
// ---------------------------------------------------------------------------
void __fastcall TForm1::ResetScripting() {
	TGeneral_RAII_HourGlass hourGlass;

	TStringList * PostData = new TStringList;
	try {
		// Reset the scripting line
		/*
		 for (int i = 1; i < 5; ++i)
		 {
		 try
		 {
		 String S = FormatFloat("000",i);
		 PostData->Clear();
		 String SS = String("edit")+S+String("=END");
		 PostData->Add(SS); // Each delete will make the first pid zero
		 PostData->Add("submit=Edit");
		 IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text + AnsiString("/script.cgi"), PostData);
		 }
		 catch(const Exception & e)
		 {
		 ShowMessage("Error encountered during POST: " + e.Message);
		 }
		 ::Sleep(250);
		 Application->ProcessMessages();
		 }
		 */
		// Reset other settings to default

		try {
			PostData->Clear();
			PostData->Add("robdelay=1");
			// PostData->Add("lcdback=5");
			// PostData->Add("submit=Submit");
			String S = IdHTTP->Post("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/script.cgi"), PostData);
		}
		catch (const Exception & e) {
			ShowMessage("Error encountered during POST Reset Scripting: " +
				e.Message);
		}
	}
	__finally {
		delete PostData;
	}
	::Sleep(500);
	Application->ProcessMessages();
}
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
void __fastcall TForm1::EdtScriptsLine1Exit(TObject * Sender) {
	TEdit * TE = dynamic_cast<TEdit*>(Sender); // For the buttons
	if (TE) {
		if (StrToInt(TE->Text) > 127 || StrToInt(TE->Text) < 0) {
			Beep();
			MessageDlg("The script must be between 000 and 127", mtError,
				TMsgDlgButtons() << mbOK, 0);
			TE->SetFocus();
		}
	}
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::BtnRunScript1Click(TObject * Sender) {
	TButton * BP = dynamic_cast<TButton*>(Sender);
	if (BP) {
		TGeneral_RAII_HourGlass hourGlass;
		if (!CompareText(L"BtnRunScript1", BP->Name)) {
			IdHTTP->Get(String("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/script?run") + EdtScriptLine1->Text + AnsiString("=run")));
		}
		if (!CompareText(L"BtnRunScript2", BP->Name)) {
			IdHTTP->Get(String("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/script?run") + EdtScriptLine2->Text + AnsiString("=run")));
		}
		if (!CompareText(L"BtnRunScript3", BP->Name)) {
			IdHTTP->Get(String("http://" + EdtURL->Text + ":" + EdtPort->Text +
				AnsiString("/script?run") + EdtScriptLine3->Text + AnsiString("=run")));
		}
	}
	Sleep(3000);
	GetStatus();
}

// ---------------------------------------------------------------------------
void __fastcall TForm1::EdtScriptLine1Exit(TObject * Sender) {
	TMaskEdit * ME = dynamic_cast<TMaskEdit*>(Sender);
	if (ME) {
		if (StrToIntDef(ME->Text, -1) > 127) {
			Beep();
			ShowMessage("Value must be 000 to 127");
			ME->SetFocus();
		}
	}

}

// ---------------------------------------------------------------------------
void __fastcall TForm1::EdtURLChange(TObject * Sender) {
	ClearAuthentication();
}
// ---------------------------------------------------------------------------
void __fastcall TForm1::CbxInvertStateClick(TObject *Sender)
{
	if (EdtURL->Text.Length() > 10 && EdtPassword->Text.Length())
		GetStatus();
}
//---------------------------------------------------------------------------

